Package: modelbpp
Title: Model BIC Posterior Probability
Version: 0.3.0
Authors@R: 
    c(person(given = "Shu Fai",
             family = "Cheung",
             role = c("aut", "cre"),
             email = "shufai.cheung@gmail.com",
             comment = c(ORCID = "0000-0002-9871-9448")),
      person(given = "Huiping",
             family = "Wu",
             role = c("aut")),
      person(given = "Shing On",
             family = "Leung",
             role = c("aut"),
             comment = c(ORCID = "0000-0002-0962-339X")),
      person(given = "Ivan Jacob Agaloos",
             family = "Pesigan",
             role = "ctb",
             email = "r.jeksterslab@gmail.com",
             comment = c(ORCID = "0000-0003-4818-8420")))
Description: Fits the neighboring models of a fitted
  structural equation model and assesses the model
  uncertainty of the fitted model based on BIC posterior
  probabilities (BPP), using the method presented in
  Wu, Cheung, and Leung (2020)
  <doi:10.1080/00273171.2019.1574546>. See
  Pesigan, Cheung, Wu, Chang, and Leung (2026)
  <doi:10.3758/s13428-025-02921-x> for an introduction to
  the package.
License: GPL (>= 3)
Encoding: UTF-8
Suggests: knitr, rmarkdown, tinytest
Depends: R (>= 4.0.0)
Imports: lavaan, parallel, pbapply, igraph, manymome
VignetteBuilder: knitr
URL: https://sfcheung.github.io/modelbpp/
BugReports: https://github.com/sfcheung/modelbpp/issues
LazyData: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-12 15:26:03 UTC; shufa
Author: Shu Fai Cheung [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9871-9448>),
  Huiping Wu [aut],
  Shing On Leung [aut] (ORCID: <https://orcid.org/0000-0002-0962-339X>),
  Ivan Jacob Agaloos Pesigan [ctb] (ORCID:
    <https://orcid.org/0000-0003-4818-8420>)
Maintainer: Shu Fai Cheung <shufai.cheung@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-12 23:40:02 UTC
Built: R 4.6.0; ; 2026-05-13 00:21:38 UTC; unix
